// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <fstream>

#include "PLF.hh"
#include "algo.hh"
#include "globals.hh"
#include "PingusError.hh"

// Create a PLF object and start parsing the given file.
PLF::PLF(string filename)
{
  x_pos = y_pos = 0;
  pingus = 100;
  music = ResDescriptor("file", "gd-walk.it");

  col = ResDescriptor("auto", "-Auto ColMap-");
  cout << "Starting Parsing" << endl;
  init(filename);
}

// Destroy all data
PLF::~PLF()
{
}

void
PLF::set_value(string valueid,
	       string cast,
	       string value)
{
  switch(current_group) {

  case PLF::GLOBAL:
    if (valueid == "start_x_pos") {
      sscanf(value.c_str(), "%d", &x_pos);
    } else if (valueid == "start_y_pos") {
      sscanf(value.c_str(), "%d", &y_pos);
    } else if (valueid == "number_of_pingus") {
      sscanf(value.c_str(), "%d", &pingus);
    } else if (valueid == "levelname") {
      levelname = value;
    } else if (valueid == "description") {
      description = value;
    } else {
      throw PingusError("Global: Unknown ValueID: " + valueid);
    }
    break;

  case PLF::BACKGROUND:
    if (valueid == "image") {
      bg.desc = ResDescriptor(cast, value);
    } else if (valueid == "scroll_x") {
      sscanf(value.c_str(), "%d", &bg.scroll_x);
    } else if (valueid == "scroll_y") {
      sscanf(value.c_str(),"%d", &bg.scroll_y);
    } else
      throw PingusError("Background: Unknown ValueID: " + valueid);
    break;

  case PLF::GROUND:
    if (valueid == "maptype") {
      if (value == "BMP") {
	maptype = BMP;
      } else if (value == "SPOT") {
	maptype = SPOT;
      } else if (value == "TILE") {
	maptype = TILE;
      } else if (value == "RANDOM") {
	maptype = RANDOM;
      } else {
	maptype = UNDEF;
      }
    } else if (valueid == "data") {
      fg = ResDescriptor(cast, value);
    } else if (valueid == "colmap") {
      col = ResDescriptor(cast, value);
    } else {
      throw PingusError("Ground: Unknown ValueID: " + valueid);
    }
    break;

  case PLF::MUSIC:
    if (valueid == "data")
      music = ResDescriptor(cast, value);
    else
      throw PingusError("Music: Unknown ValueID: " + valueid);
    break;

  case PLF::EXIT:
    if (valueid == "x_pos") {
      if (sscanf(value.c_str(), "%d", &exit_s.x_pos) != 1) cout << "Error!!!!!!" << endl;
      if (verbose > 2) cout << "Exit.x_pos: " << exit_s.x_pos << ":'" << value << "'" << endl;
    } else if (valueid == "y_pos") {
      sscanf(value.c_str(), "%d", &exit_s.y_pos);
    } else if (valueid == "z_pos") {
      sscanf(value.c_str(), "%d", &exit_s.z_pos);
      if (verbose > 2) cout << "Exit.y_pos: " << exit_s.y_pos << endl;
    } else if (valueid == "image") {
      exit_s.desc = ResDescriptor(cast, value);
      if (verbose > 2) cout << "Exit.desc: " << exit_s.desc.res_name << endl;
    } else {
      throw PingusError("Exit: Unknown ValueID: " + valueid);
    }
    break;

  case PLF::ENTRANCE:
    if (valueid == "x_pos") {
      sscanf(value.c_str(), "%d", &entrance_s.x_pos);
    } else if (valueid == "y_pos") {
      sscanf(value.c_str(), "%d", &entrance_s.y_pos);
    } else if (valueid == "z_pos") {
      sscanf(value.c_str(), "%d", &entrance_s.z_pos);
    } else if (valueid == "image") {
      entrance_s.desc.res_name = value;
    } else if (valueid == "direction") {
      if (value == "left") {
	entrance_s.direction.left();
      } else if (value == "right") {
	entrance_s.direction.right();
      } else {
	throw PingusError("Entrance: Unknown Value for direction: " + valueid);
      }
    } else if (valueid == "release_rate") {
      sscanf(value.c_str(), "%d", &entrance_s.release_rate);
    } else {
      throw PingusError("Entrance: Unknown ValueID: " + valueid);
    }
    break;

  case PLF::TRAP:
    if (valueid == "name") {
      trap_s.name = value;
    } else if (valueid == "x_pos") {
      sscanf(value.c_str(), "%d", &trap_s.x_pos);
    } else if (valueid == "y_pos") {
      sscanf(value.c_str(), "%d", &trap_s.y_pos);
    } else if (valueid == "z_pos") {
      sscanf(value.c_str(), "%d", &trap_s.z_pos);
    } else {
      throw PingusError("PLF:Trap: Unknown value id: " + valueid);      
    }
    break;
    
  case PLF::HOTSPOT:
    if (valueid == "x_pos") {
      sscanf(value.c_str(), "%d", &hotspot_s.x_pos);
    } else if (valueid == "y_pos") {
      sscanf(value.c_str(), "%d", &hotspot_s.y_pos);
    } else if (valueid == "z_pos") {
      sscanf(value.c_str(), "%d", &hotspot_s.z_pos);
    } else if (valueid == "image") {
      hotspot_s.desc = ResDescriptor(cast, value);
    } else {
      throw PingusError("PLF:Hotspot: Unknown value id " + valueid);
    }
    break;

  case PLF::BUTTONS:
    {
      button_data temp;
      temp.name = valueid;
      sscanf(value.c_str(), "%d", &temp.number_of);
      buttons.push_back(temp);
    }
    break;

  case PLF::LIQUID:
    if (valueid == "x_pos") {
      sscanf(value.c_str(), "%d", &liquid_s.x_pos);
    } else if (valueid == "y_pos") {
      sscanf(value.c_str(), "%d", &liquid_s.y_pos);
    } else if (valueid == "z_pos") {
      sscanf(value.c_str(), "%d", &liquid_s.z_pos);
    } else if (valueid == "width") {
      sscanf(value.c_str(), "%d", &liquid_s.width);
    } else if (valueid == "image") {
      liquid_s.desc = ResDescriptor(cast, value);
    } else {
      throw PingusError("PLF:Liquid: Unknown value id");
    }
    break;
    
  default:
    throw PingusError("PLF:set_value(): Internal Parser error!");
  }
}

void
PLF::set_group_start(string groupname)
{
  if (groupname == "global") {
    current_group = PLF::GLOBAL;
  } else if (groupname == "background") {
    current_group = PLF::BACKGROUND;
  } else if (groupname == "ground") {
    current_group = PLF::GROUND;
  } else if (groupname == "music") {
    current_group = PLF::MUSIC;
  } else if (groupname == "exit") {
    current_group = PLF::EXIT;
  } else if (groupname == "liquid") {
    current_group = PLF::LIQUID;
  } else if (groupname == "entrance") {
    current_group = PLF::ENTRANCE;
  } else if (groupname == "trap") {
    current_group = PLF::TRAP;
  } else if (groupname == "hotspot") {
    current_group = PLF::HOTSPOT;
  } else if (groupname == "buttons") {
    current_group = PLF::BUTTONS;
  } else {
    throw PingusError("Parse error: Unknow groupname: '" + groupname + "'");
  }  
}

void
PLF::set_group_end(void)
{
  // flush collected data
  switch(current_group) {
  case PLF::EXIT:
    exit_v.push_back(exit_s);
    exit_s.clean();
    break;

  case PLF::ENTRANCE:
    entrance_v.push_back(entrance_s);
    entrance_s.clean();
    break;

  case PLF::TRAP:
    traps.push_back(trap_s);
    trap_s.clean();
    break;

  case PLF::HOTSPOT:
    hotspot.push_back(hotspot_s);
    hotspot_s.clean();
    break;

  case PLF::LIQUID:
    liquid.push_back(liquid_s);
    liquid_s.clean();
    break;

  default:
    if (verbose > 2)
      cout << "PLF:set_group_end(): Nothing to flush" << endl;
    break;
  }

}

background_data
PLF::get_bg()
{
  return bg;
}

ResDescriptor
PLF::get_fg()
{
  ResDescriptor ret_val = fg;
  
  switch(fg.type) {
  case  ResDescriptor::FILE:
    ret_val.res_name = find_file(pingus_datadir + "levels/", fg.res_name);
    break;
  case  ResDescriptor::RESOURCE:
    // do nothing
    break;
  default:
    assert(false);
    break;
  }
  return ret_val;
}

MapType
PLF::map_type()
{
  return maptype;
}

ResDescriptor
PLF::get_mapfile()
{
  switch(col.type) {
  case  ResDescriptor::FILE:
    col.res_name = find_file(pingus_datadir + "levels/", col.res_name);
    break;
  case  ResDescriptor::RESOURCE:
    break;
  default:
    break;
  }
  return col;
}

int
PLF::get_startx()
{
  return -x_pos;
}

int
PLF::get_starty()
{
  return y_pos;
}

vector<hotspot_data>
PLF::get_hotspot()
{
  return hotspot;
}

vector<exit_data>
PLF::get_exit()
{
  return exit_v;
}

vector<entrance_data>
PLF::get_entrance()
{
  return entrance_v;
}

vector<trap_data> 
PLF::get_traps()
{
  return traps;
}

int
PLF::get_pingus()
{
  return pingus;
}

vector<button_data>
PLF::get_buttons()
{
  return buttons;
}

ResDescriptor
PLF::get_music()
{
  return music;
}

string
PLF::get_description(void)
{
  return description;
}

string
PLF::get_levelname()
{
  return levelname;
}

vector<liquid_data>
PLF::get_liquids(void)
{
  return liquid;
}

/* EOF */
